/*
 * mcformat.c -- PlayStaion 2 memory card initialize command
 *
 *	Copyright (C) 2000  Sony Computer Entertainment Inc.
 *
 *  This file is subject to the terms and conditions of the GNU General
 *  Public License Version 2. See the file "COPYING" in the main
 *  directory of this archive for more details.
 */

#include <stdio.h>
#include <getopt.h>
#include <asm/types.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <linux/ps2/mcio.h>

int override_mode = 0;
int query_mode = 0;
int show_help = 0;
int do_unformat = 0;
int do_hardformat = 0;
int builtin_icon = 0;
char *skeldir = "/etc/ps2mcskel";

#define ICONSYSNAME	"/icon.sys"
#define ICONDATANAME	"/penguin.ico"
unsigned char icon_sys[] = {
#include "icon_sys.inc"
};
unsigned char icon_data[] = {
#include "ps2linux_ico.inc"
};

static struct option const long_options[] =
{
	{"unformat",	no_argument,		NULL,		'u'	},
	{"hardformat",	no_argument,		NULL,		'h'	},
	{"force",	no_argument,		NULL,		'f'	},
	{"query",	no_argument,		NULL,		'q'	},
	{"showstatus",	no_argument,		NULL,		'q'	},
	{"format",	no_argument,		&do_unformat,	0	},
	{"help",	no_argument,		&show_help,	1	},
	{"skeleton",	required_argument,	NULL,		'd'	},
	{NULL,		0,			NULL,		0	}
};

usage(int es)
{
  fprintf(stderr,
	  "usage: mcformat [options] device_name\n"
	  "    or mcformat [-f] -u device_name\n"
	  "    or mcformat -q device_name\n"
	  "initialize PlayStation 2 memory card.\n"
	  "\n"
	  "      --format       create directory for linux (this is default)\n"
	  "  -u, --unformat     unformat memory card (all data will be lost)\n"
	  "  -h, --hardformat   format memory card entirely (all data will be lost)\n"
	  "  -f, --force        format even if the device is busy\n"
	  "  -q, --query        show device status\n"
	  "  -d, --skeleton=dir specify directory which contains initial files \n"
	  "                     (default is /etc/ps2mcskel)\n"
	  "      --showstatus   same as -q\n"
	  "      --help         display this help and exit\n"
	  );
  exit (es);
}

char *
gettypename(int type)
{
	switch (type) {
	case PS2MC_TYPE_EMPTY:
		return ("EMPTY");
	case PS2MC_TYPE_PS1:
		return ("PS1");
	case PS2MC_TYPE_PS2:
		return ("PS2");
	case PS2MC_TYPE_POCKETSTATION:
		return ("PocketStation");
	}
	return ("unknown");
}

int
write_data(int fd, char *path, unsigned char *data, int count)
{
	struct ps2mc_arg cmdarg;

	memset(&cmdarg, 0, sizeof(cmdarg));
	cmdarg.path = path;
	cmdarg.pathlen = strlen(cmdarg.path);
	cmdarg.mode = (O_WRONLY | O_CREAT);
	cmdarg.pos = 0;
	cmdarg.data = data;
	cmdarg.count = count;
	return ioctl(fd, PS2MC_IOCWRITE, &cmdarg);
}

int
write_file(int srcfd, int destfd, char *path)
{
	struct ps2mc_arg cmdarg;
	unsigned char buf[1024*16];
	int n, pos, res;

	memset(&cmdarg, 0, sizeof(cmdarg));
	cmdarg.path = path;
	cmdarg.pathlen = strlen(cmdarg.path);
	cmdarg.mode = (O_WRONLY | O_CREAT);

	pos = 0;
	while (0 < (n = read(srcfd, buf, sizeof(buf)))) {
		cmdarg.pos = pos;
		cmdarg.data = buf;
		cmdarg.count = n;
		if (ioctl(destfd, PS2MC_IOCWRITE, &cmdarg) < 0)
			return (-1);
		pos += n;
	}
	return (n); /* n == 0 succeeded,  n < 0 error */
}

main(int ac, char *av[])
{
	int c, skeldirlen;
	int mcfd, fd;
	struct ps2mc_cardinfo cardinfo;
	struct stat filestat;
	DIR *dir;
	struct dirent *dirent;
	char path[MAXPATHLEN];

	while ((c = getopt_long (ac, av, "dhfqu", long_options, (int *) 0))
	       != EOF) {
		switch (c) {
		case 0:
			break;
		case 'f':
			override_mode = 1;
			break;
		case 'q':
			query_mode = 1;
			break;
		case 'u':
			do_unformat = 1;
			break;
		case 'h':
			do_hardformat = 1;
			break;
		case 'd':
			if (optarg) {
				skeldir = optarg;
			} else {
				if (ac <= optind)
					usage(1);
				skeldir = av[optind++];
			}
			break;
		default:
			usage (1);
	  }
	}

	if (ac <= optind)
		usage(1);

	if (show_help)
		usage(0);

	if ((mcfd = open(av[optind], O_RDWR)) < 0) {
		perror(av[optind]);
		exit(1);
	}

	if (fstat(mcfd, &filestat) < 0) {
		perror("fstat()");
		exit(1);
	}

	if (!S_ISBLK(filestat.st_mode)) {
		fprintf(stderr, "%s isn't a block device.\n", av[optind]);
		exit(1);
	}

	if (ioctl(mcfd, PS2MC_IOCGETINFO, &cardinfo) < 0) {
		perror("ioctl(GETINFO)");
		exit(1);
	}
  
	if (query_mode) {
		printf("port %d, slot %d: %s %s\n",
		       PS2MC_PORT(filestat.st_rdev),
		       PS2MC_SLOT(filestat.st_rdev),
		       gettypename(cardinfo.type),
		       cardinfo.type == PS2MC_TYPE_PS2 ?
		       (cardinfo.formatted?"formatted":"unformatted") : "");
		exit (0);
	}

	if (1 < cardinfo.busy && !override_mode) {
		fprintf(stderr, "%s: Device busy\n", av[optind]);
		exit(1);
	}

	skeldirlen = strlen(skeldir);
	if (skeldirlen == 0) {
		skeldir = ".";
		skeldirlen = 1;
	} else {
		if (skeldir[skeldirlen - 1] == '/') {
			skeldirlen -= 1;
			skeldir[skeldirlen] == '\0';
		}
	}

	if (do_unformat) {
		fprintf(stdout, "unformatting %s...", av[optind]);
		fflush(stdout);
		if (ioctl(mcfd, PS2MC_IOCUNFORMAT) < 0) {
			fprintf(stdout, "\n");
			perror("unformat");
			exit(1);
		}
		fprintf(stdout, "done\n");
	} else {
		fprintf(stdout, "formatting %s...", av[optind]);
		fflush(stdout);

		/*
		 * format
		 */
		if (do_hardformat && ioctl(mcfd, PS2MC_IOCFORMAT) < 0) {
			fprintf(stdout, "\n");
			perror("format");
			exit(1);
		}
		if (ioctl(mcfd, PS2MC_IOCSOFTFORMAT) < 0) {
			fprintf(stdout, "\n");
			perror("format");
			exit(1);
		}

		/*
		 * copy skeleton
		 */
		if ((dir = opendir(skeldir)) == NULL) {
			fprintf(stdout, "\n");
			perror(skeldir);
			builtin_icon = 1;
			goto write_builtin_icon;
		}
		while ((dirent = readdir(dir)) != NULL) {
			if (strcmp(dirent->d_name, ".") == 0 ||
			    strcmp(dirent->d_name, "..") == 0)
				continue;
			sprintf(path, "%s/%s", skeldir, dirent->d_name);
			if ((fd = open(path, O_RDONLY)) < 0) {
				fprintf(stdout, "\n");
				perror("penguin.ico");
				builtin_icon = 1;
				closedir(dir);
				goto write_builtin_icon;
			}
			if (write_file(fd, mcfd, &path[skeldirlen]) < 0) {
				fprintf(stdout, "\n");
				perror("write");
				close(fd);
				builtin_icon = 1;
				closedir(dir);
				goto write_builtin_icon;
			}
			close(fd);
		}
		closedir(dir);

	write_builtin_icon:
		/*
		 * copy builtin icon files if you failed to copy skeleton
		 */
		if (builtin_icon) {
			if (ioctl(mcfd, PS2MC_IOCSOFTFORMAT) < 0) {
				fprintf(stdout, "\n");
				perror("format");
				exit(1);
			}
			if (write_data(mcfd, ICONSYSNAME,
				       icon_sys, sizeof(icon_sys)) < 0) {
				fprintf(stdout, "\n");
				perror("write");
			}
			if (write_data(mcfd, ICONDATANAME,
				       icon_data, sizeof(icon_data))<0) {
				fprintf(stdout, "\n");
				perror("write");
			}
		}

		fprintf(stdout, "done\n");
	}

	ioctl(mcfd, PS2MC_IOCNOTIFY);

	close(mcfd);
	exit (0);
}
