/*
 *  setcrtmode.c -- PlayStation 2 screen mode initialize command
 *
 *	Copyright (C) 2000  Sony Computer Entertainment Inc.
 *
 *  This file is subject to the terms and conditions of the GNU General
 *  Public License Version 2. See the file "COPYING" in the main
 *  directory of this archive for more details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <linux/ps2/dev.h>
#include <linux/ps2/gs.h>

#define DEFAULTCMD	"/etc/crtmode.default"

int gsfd;
char *cmdfile = NULL;

struct option long_options[] = {
    { "help",	    no_argument,	NULL,	    'h' },
    { "print",	    no_argument,	NULL,	    'p' },
    { "file",	    required_argument,	NULL,	    'f' },
};

char *videomode[] = { "VESA", "DTV", "NTSC", "PAL", NULL };
char *interlace[] = { "nointerlace", "interlace", NULL };
char *resolution[] = { "640x480", "800x600", "1024x768", "1280x1024", NULL };
char *dtvmode[] = { "480P", "1080I", "720P", NULL };
char *refreshrate[] = { "75Hz", "60Hz", NULL };
char *pixelfmt[] = { "32bpp", "24bpp", "16bpp", NULL };

struct { char *name; int mode; int res; } moderes[] = {
    { "ntsc0", PS2_GS_NTSC, PS2_GS_NOINTERLACE },
    { "ntsc1", PS2_GS_NTSC, PS2_GS_INTERLACE },
    { "pal0", PS2_GS_PAL, PS2_GS_NOINTERLACE },
    { "pal1", PS2_GS_PAL, PS2_GS_INTERLACE },
    { "vesa0", PS2_GS_VESA, PS2_GS_640x480 },
    { "vesa1", PS2_GS_VESA, PS2_GS_800x600 },
    { "vesa2", PS2_GS_VESA, PS2_GS_1024x768 },
    { "vesa3", PS2_GS_VESA, PS2_GS_1280x1024 },
    { "dtv0", PS2_GS_DTV, PS2_GS_480P },
    { "dtv1", PS2_GS_DTV, PS2_GS_1080I },
    { "dtv2", PS2_GS_DTV, PS2_GS_720P },
    { NULL, 0, 0 }
};

struct { int w, h; } reslist[4][4] = {
    { { 640, 480 }, { 800, 600 }, { 1024, 768 }, { 1280, 1024 }, }, 
    { { 720, 480 }, { 1920, 1080 }, { 1280, 720 }, { -1, -1 }, }, 
    { { 640, 224 }, { 640, 448 }, { -1, -1 }, { -1, -1 }, },
    { { 640, 240 }, { 640, 480 }, { -1, -1 }, { -1, -1 }, },
};

void usage(void)
{
    printf(
	"Usage: setcrtmode [options] [parameters]..\n"
	"Set PlayStation 2 screen mode\n"
	"\n"
	"  -f setting-file, --file=setting-file\n"
	"                        read setting from file\n"
	"  -p, --print           print out current screen mode\n"
	"  -h, --help            display this help and exit\n"
	"\n"
	"Parameters:\n"
	"  VESA, DTV, NTSC, PAL\n"
	"                        CRT output signal mode\n"
	"  640x480, 800x600, 1024x768, 1280x1024\n"
	"                        VESA resolution type\n"
	"  60Hz, 75Hz\n"
	"                        VESA screen refresh rate\n"
	"  480P, 1080I, 720P\n"
	"                        DTV resolution type\n"
	"  nointerlace, interlace\n"
	"                        NTSC, PAL interlace mode\n"
	"  32bpp, 24bpp, 16bpp\n"
	"                        framebuffer pixel format\n"
	"  vesa0[,{60|75}]       same as 'VESA 640x480 [60Hz|75Hz]'\n"
	"  vesa1[,{60|75}]               'VESA 800x600 [60Hz|75Hz]'\n"
	"  vesa2[,{60|75}]               'VESA 1024x768 [60Hz|75Hz]'\n"
	"  vesa3[,{60|75}]               'VESA 1280x1024 [60Hz|75Hz]'\n"
	"  dtv0                          'DTV 480P'\n"
	"  dtv1                          'DTV 1080I'\n"
	"  dtv2                          'DTV 720P'\n"
	"  ntsc0                         'NTSC nointerlace'\n"
	"  ntsc1                         'NTSC interlace'\n"
	"  pal0                          'PAL nointerlace'\n"
	"  pal1                          'PAL interlace'\n"
    );
    exit(1);
}

void print(struct ps2_screeninfo *info)
{
    switch (info->mode) {
    case PS2_GS_VESA:
	printf("%s %s %s %s %d %d\n",
	       videomode[info->mode & 3],
	       resolution[info->res & 3],
	       refreshrate[(info->res >> 8) & 1],
	       pixelfmt[info->psm & 3],
	       info->w, info->h);
	break;
    case PS2_GS_DTV:
	printf("%s %s %s %d %d\n",
	       videomode[info->mode & 3],
	       dtvmode[info->res & 3],
	       pixelfmt[info->psm & 3],
	       info->w, info->h);
	break;
    case PS2_GS_NTSC:
    case PS2_GS_PAL:
	printf("%s %s %s %d %d\n",
	       videomode[info->mode & 3],
	       interlace[info->res & 1],
	       pixelfmt[info->psm & 3],
	       info->w, info->h);
	break;
    }
}

int match(char **strs, char *arg)
{
    int i;

    for (i = 0; strs[i] != NULL; i++)
	if (strcasecmp(strs[i], arg) == 0)
	    return i;
    return -1;
}

int parse(int oi, int ac, char **av, struct ps2_screeninfo *info)
{
    int i, j;
    int r;
    char *p, *q, c;
    int wh = 0;

    for (i = oi; i < ac; i++) {
	p = av[i];
	if ((r = match(videomode, p)) >= 0) {
	    info->mode = r;
	    info->res = info->w = info->h = -1;
	} else if ((r = match(interlace, p)) >= 0) {
	    if (info->mode < 0)
		info->mode = PS2_GS_NTSC;
	    else if (info->mode != PS2_GS_NTSC && info->mode != PS2_GS_PAL)
		return i;
	    info->res = r;
	    info->w = info->h = -1;
	} else if ((r = match(resolution, p)) >= 0) {
	    if (info->mode < 0)
		info->mode = PS2_GS_VESA;
	    else if (info->mode != PS2_GS_VESA)
		return i;
	    info->res = r;
	    info->w = info->h = -1;
	} else if ((r = match(refreshrate, p)) >= 0) {
	    if (info->mode < 0)
		info->mode = PS2_GS_VESA;
	    else if (info->mode != PS2_GS_VESA)
		return i;
	    if (info->res < 0)
		info->res = PS2_GS_640x480;
	    info->res &= 0xff;
	    info->res |= r << 8;
	} else if ((r = match(dtvmode, p)) >= 0) {
	    if (info->mode < 0)
		info->mode = PS2_GS_DTV;
	    else if (info->mode != PS2_GS_DTV)
		return i;
	    info->res = r;
	    info->w = info->h = -1;
	} else if ((r = match(pixelfmt, p)) >= 0) {
	    info->psm = r;
	} else if (wh == 0 && sscanf(p, "%d%c", &r, &c) == 1 && r > 0) {
	    info->w = r;
	    wh = i;
	} else if (wh != 0 && sscanf(p, "%d%c", &r, &c) == 1 && r > 0) {
	    info->h = r;
	    wh = 0;
	} else {
	    for (j = 0; moderes[j].name != NULL; j++) {
		if (strncasecmp(p, moderes[j].name, strlen(moderes[j].name)))
		    continue;
		info->mode = moderes[j].mode;
		info->res = moderes[j].res;
		info->w = info->h = -1;
		q = p + strlen(moderes[j].name);
		if (*q != '\0') {
		    if (info->mode != PS2_GS_VESA)
			return i;
		    if (sscanf(q, ",%d%c", &r, &c) != 1)
			return i;
		    if (r == 60)
			info->res |= PS2_GS_60Hz;
		    else if (r == 75)
			info->res |= PS2_GS_75Hz;
		    else
			return i;
		}
		break;
	    }
	    if (moderes[j].name == NULL)
		return i;
	}
    }
    if (wh != 0)
	return wh;
    return 0;
}

int readfile(char *file, struct ps2_screeninfo *info)
{
    char buf[256];
    FILE *fp;
    int result;
    char *av[128];
    int ac;
    char *p;

    if (strcmp("-", file) == 0) {
	fp = stdin;
    } else {
	if ((fp = fopen(file, "r")) == NULL) {
	    if (file != DEFAULTCMD) {
		perror(file);
		return -1;
	    }
	    return 0;
	}
    }

    while (fgets(buf, sizeof(buf), fp) != NULL) {
	p = buf;
	ac = 1;
	do {
	    while (*p != '\0' && strchr(" \t\r\n", *p) != NULL)
		*p++ = '\0';
	    if (*p == '\0')
		break;
	    av[ac++] = p;
	    while (*p != '\0' && strchr(" \t\r\n", *p) == NULL)
		p++;
	} while (*p != '\0');
	if (ac > 1 && av[1][0] != '#') {
	    if ((result = parse(1, ac, av, info)) != 0) {
		fprintf(stderr, "setcrtmode: invalid argument -- %s\n",
		    av[result]);
		fclose(fp);
		return -1;
	    }
	}
    }
    fclose(fp);
    return 1;
}

int main(int argc, char **argv)
{
    int do_help = 0, do_print = 0;
    int c;
    int result, getmode;
    struct ps2_screeninfo info;

    while ((c = getopt_long(argc, argv, "hpf:", long_options, NULL)) != EOF) {
	switch (c) {
	case 'h':
	    do_help = 1;
	    break;
	case 'p':
	    do_print = 1;
	    break;
	case 'f':
	    cmdfile = optarg;
	    break;
	default:
	    usage();
	}
    }

    if (do_help)
	usage();

    if (argc <= optind && cmdfile == NULL)
	cmdfile = DEFAULTCMD;

    info.mode = info.res = info.w = info.h = info.fbp = info.psm = -1;
    getmode = 0;
    if (!do_print && cmdfile != NULL) {
	if((getmode = readfile(cmdfile, &info)) < 0)
	    exit(1);
    }

    if ((gsfd = open(PS2_DEV_GS, O_RDWR)) < 0) {
	perror(PS2_DEV_GS);
	exit(1);
    }

    if (!getmode && ioctl(gsfd, PS2IOC_GSCREENINFO, &info) < 0) {
	close(gsfd);
	perror(argv[0]);
	exit(1);
    }

    if (do_print) {
	print(&info);
    } else {
	if ((result = parse(optind, argc, argv, &info)) != 0) {
	    fprintf(stderr, "%s: invalid argument -- %s\n",
		argv[0], argv[result]);
	    close(gsfd);
	    exit(1);
	}

	if (info.mode < 0)
	    info.mode = PS2_GS_VESA;
	if (info.res < 0)
	    if (info.mode == PS2_GS_VESA)
		info.res = PS2_GS_640x480;
	    else if (info.mode == PS2_GS_DTV)
		info.res = PS2_GS_480P;
	    else
		info.res = PS2_GS_INTERLACE;
	if (info.w <= 0 || info.h <= 0) {
	    info.w = reslist[info.mode][info.res & 0xff].w;
	    info.h = reslist[info.mode][info.res & 0xff].h;
	}
	info.fbp = 0;

	if (info.psm < 0)
	    info.psm = PS2_GS_PSMCT32;
	if (info.w * info.h > 1024 * 1024)
	    info.psm = PS2_GS_PSMCT16;

	if (ioctl(gsfd, PS2IOC_SSCREENINFO, &info) != 0) {
	    fprintf(stderr, "%s: illegal screen mode\n", argv[0]);
	    close(gsfd);
	    exit(1);
	}
    }

    close(gsfd);
    return 0;
}
